unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Buttons, StdCtrls;

type
  TFormFont = class(TForm)
    Panel1: TPanel;
    Box: TPaintBox;
    Timer1: TTimer;
    Panel2: TPanel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    ODialog: TOpenDialog;
    SDialog: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SpeedButton4Click(Sender: TObject);
    procedure SpeedButton5Click(Sender: TObject);
    procedure BoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure BoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormFont: TFormFont;

implementation

{$R *.dfm}

type TMKFont = array[0..191,0..6] of Byte;

var BitMap: TBitMap;
    Fnt: TMKFont;
    CNm: Byte;
    MD: Boolean;
    But: TMouseButton;

procedure ClearBitMap;
//  :
var X, Y: Integer;
begin
  with BitMap.Canvas do
  begin
    Pen.Color := clRed;
    Brush.Color := clWhite;
    Rectangle (0, 0, BitMap.Width, BitMap.Height);
    for X := 0 to 5 do
    begin
      MoveTo (X * 16, 0);
      LineTo (X * 16, BitMap.Height);
    end; {for}
    for Y := 0 to 7 do
    begin
      MoveTo (0, Y * 16);
      LineTo (BitMap.Width, Y * 16);
    end; {for}
  end; {with}
end; {proc ClearBitMap}

procedure View;
//   :
var X, Y: Integer;
    C: Byte;
begin
  ClearBitMap;
  with FormFont do
  begin
    StaticText1.Caption := Concat (' ', IntToStr (CNm));
    StaticText2.Caption := Concat (' ', Chr (CNm));
  end; {with}
  with BitMap.Canvas do
  begin
    for Y := 0 to 6 do
    begin
      C := Fnt[CNm,Y];
      X := 0;
      while C <> 0 do
      begin
        if Odd (C) then Brush.Color := clBlack
                   else Brush.Color := clWhite;
        FloodFill (X * 16 + 8, Y * 16 + 8, clRed, fsBorder);
        Inc (X);
        C := C shr 1;
      end; {while}
    end; {for}
  end; {with}
end; {proc View}

procedure ClearFont;
//  :
var I, J: Integer;
begin
  for I := 0 to 191 do
    for J := 0 to 6 do
      Fnt[I,J] := 0;
  CNm := 32;
  View;
end; {proc ClearFont}

procedure OpenFont (FName: string);
//  :
var F: file of Byte;
    I, J: Byte;
begin
  {$I-}
  AssignFile (F, FName);
  Reset (F);
  for I := 0 to 191 do
    for J := 0 to 6 do
      Read (F, Fnt[I,J]);
  CloseFile (F);
  {$I+}          
  if IOResult <> 0
    then MessageDlg ('    .', mtError, [mbOk], 0);
end; {proc OpenFont}


procedure TFormFont.FormCreate(Sender: TObject);
var S: string;
begin
  BitMap := TBitMap.Create;
  BitMap.Width := Box.Width;
  BitMap.Height := Box.Height;
  ClearFont;
  Timer1.Enabled := True;
  GetDir (0, S);
  SDialog.InitialDir := S;
  ODialog.InitialDir := S;
  MD := False;
end;

procedure TFormFont.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if MessageDlg (' ,   ?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then
  begin
    BitMap.Free;
  end else Abort;
end;

procedure TFormFont.Timer1Timer(Sender: TObject);
begin
  Box.Canvas.Draw (0, 0, BitMap);
end;

procedure TFormFont.SpeedButton3Click(Sender: TObject);
begin
  if MessageDlg (' ,    ?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then ClearFont;
end;

procedure TFormFont.SpeedButton2Click(Sender: TObject);
begin
  if CNm < 191 then
  begin
    Inc (CNm);
    View;
  end; {if}
end;

procedure TFormFont.SpeedButton1Click(Sender: TObject);
begin
  if CNm > 0 then
  begin
    Dec (CNm);
    View;
  end; {if}
end;

procedure TFormFont.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = vk_Left then SpeedButton1Click (nil);
  if Key = vk_Right then SpeedButton2Click (nil);
end;

procedure TFormFont.SpeedButton4Click(Sender: TObject);
var F: file of Byte;
    I, J: Byte;
begin
  if SDialog.Execute then
  begin
    {$I-}
    AssignFile (F, SDialog.FileName);
    Rewrite (F);
    for I := 0 to 191 do
      for J := 0 to 6 do
        Write (F, Fnt[I,J]);
    CloseFile (F);
    {$I+}
    if IOResult <> 0
      then MessageDlg ('   .', mtError, [mbOk], 0);
  end; {if}
end;

procedure TFormFont.SpeedButton5Click(Sender: TObject);
begin
  if ODialog.Execute then
  begin
    OpenFont (ODialog.FileName);
    CNm := 32;
    View;
  end; {if}
end;

procedure TFormFont.BoxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MD := True;
  But := Button;
  BoxMouseMove (nil, Shift, X, Y);
end;

procedure TFormFont.BoxMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var MX, MY: Integer;
var A: Byte;
begin
  if (MD) and (X > 0) and (Y > 0) and (X < Box.Width) and (Y < Box.Height) then
  begin
    MX := X div 16;
    MY := Y div 16;
    A := 1 shl MX;
    if But = mbLeft then
    begin
      Fnt[CNm,MY] := Fnt[CNm,MY] or A;
    end else
    begin
      A := 255 xor (1 shl MX);
      Fnt[CNm,MY] := Fnt[CNm,MY] and A;
    end; {if}
    View;
  end; {if}
end;

procedure TFormFont.BoxMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MD := False;
end;

end.
